# -*- coding: utf-8 -*-

"""Top-level package for heat1d."""

__author__ = """Paul O. Hayne"""
__email__ = 'paul.hayne@lasp.colorado.edu'
__version__ = '0.1.4'

# pull everything from the original heat1d module
# into the main namespace
from .main import *
